@@ -1,32 +1,39 @@
 /*
-  Xquote Copyright (c)1997 Mark Buser,
-  All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-  
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xquote, but
-  modified versions may not be distributed
-  without prior consent of the author.
-  
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 1.12 $ $Date: 1997/10/31 23:08:04 $
-*/
-
+ * Copyright (c) 1997, 1999, 2000, 2001, Mark Buser.
+ * Copyright (c) 2001, 2002, 2003, 2004, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/optnet.c,v 1.41 2004/12/23 19:55:42 danny Exp $
+ */
 #include <sys/types.h>
 #include <stdlib.h>
-#include <arpa/inet.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <sys/socket.h>
 #include <netdb.h>
 #include <errno.h>
@@ -43,34 +50,32 @@
 #include "opttick.h"
 #include "optupdate.h"
 #include "parse.h"
-#include "remote.h"
+#include "remoteXq.h"
 #include "server.h"
 #include "status.h"
 #include "tape.h"
 #include "xutil.h"
 #include "xquote.h"
 
-/* Extern */
-extern Widget Toplevel;
-extern char time_msg[80];
+#ifdef XQUOTE
+#include "xquote.h"
+#else
+#include "xinvest.h"
+#endif
+
+#define	REREAD_LMT	4
 
 /* Globals */
 int netFinished = True;
-XtWorkProcId work;
-
-/* Local Globals */
-Widget optmenu;
-
-/* Pending vars */
-static int pServer;
+XtWorkProcId work = (XtWorkProcId)NULL;
+int	reread_lmt = REREAD_LMT;
 
 /*
 ** Net option support
 */
-
 static void netTellXinvest ()
 {
-  Display *dpy = XtDisplay (Toplevel);
+  Display *dpy = XtDisplay (per->Toplevel);
   int i;
   char *price;
   char date[11];
@@ -84,24 +89,36 @@ static void netTellXinvest ()
 
 #if !defined (NO_XMU)
   write_status_line ("Contacting Xinvest...");
-  XmUpdateDisplay(Toplevel);
+  XmUpdateDisplay(per->Toplevel);
 
   /* 
   ** Send "NAV ticker mm/dd/yyyy fffff.fff" for each ticker.
   */ 
   for (i=0; i < tickGetNum(); i++) {
     QUERY_STRUCT *data = tickGetDetail(i);
-    if (data && data->values[PRICE]) {
+    if (data && data->values[DETAIL_PRICE]) {
 
-      price = data->values[PRICE];
+      price = data->values[DETAIL_PRICE];
       if (strcmp (price, "N/A") == 0) {     /* no price */
         continue;
       } else if (strchr (price, '/')) {     /* fraction or decimal */
 	int whole, num, den;
 	sscanf( price, "%d %d / %d", &whole, &num, &den);
 	nav = (double)whole + ((double)num/(double)den);
-      } else
-	nav = strtod (price, NULL);
+      } else {
+	char *p;
+	/* Try to figure out European/American number notation */
+	if (strchr(price, '.')) {		/* American notation */
+		nav = strtod (price, NULL);
+	} else if (p = strchr(price, ',')) {	/* Europe */
+		*p = '.';
+		nav = strtod (price, NULL);
+		*p = ',';
+	} else {				/* Dunno, don't bother */
+		nav = strtod (price, NULL);
+	}
+	
+      }
 
       /* Dates from the net could be anything, punt and use today */
       strftime ( date, 11, "%m/%d/%Y", now);
@@ -125,44 +142,84 @@ static void netTellXinvest ()
 #endif
 }
 
-static char *makeQuery (int proxy, int tick)
+char *makeQuery (int proxy, int tick)
 {
-  char *url, *name;
-  char *insert, *nohost;
+  char *url, *name, *x = NULL;
+  char *insert, *nohost, *host;
   static char buf[BUFSIZ];
 
   if (tick != WORK_NEWS) {
-    setServerType (tickGetType(tick));
-    url = getServer(URL);
+    setCurServer (tickGetServer(tick));
+    setCurServerType (tickGetType(tick));
     name = tickGetName (tick);
   } else {
-    url = "sunsite.auc.dk/SYMBOL/";
-    name = "xinvest";
+    setCurServer (NEWS);
+    setCurServerType (0);
+    name = "";
   }
+  url = getServer(DETAIL_URL);
   insert = strstr (url, "SYMBOL");
   nohost = strchr (url, '/');
-  if (insert && nohost) {
 
+  host = malloc(strlen(url)+2);
+  strcpy(host, url);
+  x = strchr(host, '/');
+  if (x) *x = '\0';
+  x = NULL;
+
+#ifdef DEBUG
+fprintf(stderr, "makeQuery(%s,%s)\n", url, name);
+#endif
+
+  if (strchr(name, ' ')) {
+	int	i, cnt, j;
+	for (i=0,cnt=0; name[i]; i++)
+		if (name[i] == ' ')
+			cnt++;
+	x = malloc(i+2*cnt+8);
+	for (i=0,j=0; name[i]; i++) {
+		if (name[i] == ' ') {
+			x[j++] = '%';
+			x[j++] = '2';
+			x[j++] = '0';
+		} else {
+			x[j++] = name[i];
+		}
+	}
+	x[j] = 0;
+  }
+  if (insert && nohost) {
     if (proxy)
       sprintf ( buf, "GET http://%.*s%s%s HTTP/1.0\r\n"
-                     "User-Agent: Xquote/1.0\r\n"
+			"Host: %s\r\n"
+                     "User-Agent: Xquote/2.0\r\n"
                      "Accept: */*\r\n"
                      "\r\n",
-  	        insert-url, url, name, insert + strlen("SYMBOL") );
+  	        insert-url, url,
+		x ? x : name,
+		insert + strlen("SYMBOL"),
+		host);
     else
       sprintf ( buf, "GET %.*s%s%s HTTP/1.0\r\n"
-                     "User-Agent: Xquote/1.0\r\n"
+			"Host: %s\r\n"
+                     "User-Agent: Xquote/2.0\r\n"
                      "Accept: */*\r\n"
                      "\r\n",
-  	        insert-nohost, nohost, name, insert + strlen("SYMBOL") );
+  	        insert-nohost, nohost,
+		x ? x : name,
+		insert + strlen("SYMBOL"),
+		host);
 
   } else 
     strcpy (buf, "");
 
-#if 0
+#ifdef DEBUG
   fprintf (stderr, buf);
 #endif
 
+  if (x) free(x);
+  free(host);
+
   return (buf);
 }
 
@@ -193,8 +250,8 @@ static int readQuery(int s, char **html)
     }
   }
 
-#if 0
-  fprintf (stderr, buf);
+#ifdef DEBUG
+  if (*html) fprintf (stderr, "ReadQuery(%s) -> [%s]\n", *html, buf);
 #endif
 
   return (num);
@@ -229,20 +286,36 @@ int workNet (int command)
   static int port;
 
   static char *html;
+  static int reread_cnt;
 
   if (command == WORK_CLEAN)
     state = END;
 
-  AllowShellResize (Toplevel, DISALLOW);
+  if (command == UPDATE)
+	  state = UPDATE;
+
+  AllowShellResize (per->Toplevel, DISALLOW);
+#ifdef DEBUG
+  fprintf(stderr, "workNet(%s)\n",
+		  (command == WORK_CLEAN) ? "WORK_CLEAN" :
+		  (state == IDLE) ? "IDLE" :
+		  (state == SOCKET) ? "SOCKET" :
+		  (state == CONNECT) ? "CONNECT" :
+		  (state == WRITE) ? "WRITE" :
+		  (state == READ) ? "READ" :
+		  (state == UPDATE) ? "UPDATE" :
+		  (state == END) ? "END" :
+		  (state == PARSE) ? "PARSE" :
+		  (state == TEST) ? "TEST" : "??");
+#endif
 
   switch (state) {
 
     case IDLE:
       cylonStart (True);
 
-      /* This assumes the host is the same regardless of the server type */
-      i = 0;
-      setServerType (tickGetType(i));
+      setCurServer ( (command==WORK_NEWS)?NEWS:tickGetServer(i) );
+      setCurServerType ( (command==WORK_NEWS)?0:tickGetType(i) );
 
       /* 
       ** Determine server and port
@@ -250,13 +323,9 @@ int workNet (int command)
       proxy = True;
       getProxy (&serv_machine, &port);
       if (serv_machine == NULL) {                    /* No proxy */
-        if (command == WORK_NEWS) {
-          serv_machine = XtNewString ("sunsite.auc.dk");
-        } else {
-          size = strcspn ( getServer(URL), "/");     /* Get machine from URL */
-          serv_machine = XtCalloc ( size+1, sizeof(char));
-          sprintf ( serv_machine, "%.*s", size, getServer(URL));
-        }
+        size = strcspn ( getServer(DETAIL_URL), "/");       /* Get machine from URL */
+        serv_machine = XtCalloc ( size+1, sizeof(char));
+        sprintf ( serv_machine, "%.*s", size, getServer(DETAIL_URL));
 
         if ( (sp = getservbyname ("http", "tcp")) )  /* Get port from lookup */
           port = ntohs(sp->s_port);
@@ -351,10 +420,11 @@ int workNet (int command)
         state = WRITE;
 
       if (state == WRITE) {
-        if (command == WORK_NEWS)
+        if (command == WORK_NEWS) {
 	  html = makeQuery ( proxy, WORK_NEWS);
-        else
+	} else {
 	  html = makeQuery ( proxy, i);
+	}
       }
 
       return (False);
@@ -374,8 +444,7 @@ int workNet (int command)
 	  state = END;
 	}
       } else if (status == strlen(html)) {  /* DONE */
-        sprintf ( errmsg, "Reading %s...", 
-		  (command!=WORK_NEWS) ? tickGetName(i):"Xquote News");
+        sprintf ( errmsg, "Reading %s...", (command!=WORK_NEWS) ? tickGetName(i):"Xquote News");
         write_status_line (errmsg);
         html = NULL;
         state = READ;
@@ -384,11 +453,13 @@ int workNet (int command)
 	state = WRITE;
       }
 
+      /* Set last update to current time */
+      set_current_time();
+
       return (False);
       /* break; */
 
     case READ:
-
       /* Read the query response */
       status = readQuery (s, &html);
       if (status < 0) {                  /* ERROR */
@@ -410,16 +481,56 @@ int workNet (int command)
 	if (html) {
 	  sscanf ( html, "HTTP/%*f %d %n", &code, &num);
 	  if (code != 200) {
-	    start = html + num;
-	    if ( (end = strchr ( start, '\r')) == NULL)
-	      end = start + 20;
-	    sprintf ( errmsg, "%s: %d %.*s", 
-		     tickGetName(i), code, end-start, start);
-	    write_status (errmsg, ERR);
+		  if (code == 400) {
+			  /* attempt to deal with code 400 bad request by re-reading */
+			  if ( reread_cnt >= reread_lmt ) {    /* enough already give it up */
+				  start = html + num;
+				  if ( (end = strchr ( start, '\r')) == NULL)
+					  end = start + 20;
+				  sprintf ( errmsg, "%s: re-read limit reached on %d %.*s", 
+					  (command!=WORK_NEWS) ? tickGetName(i):"Xquote News",
+					code, end-start, start);
+				  write_status (errmsg, ERR);
+				  close(s);
+				  state = TEST;
+				  XtFree(html);
+				  html = NULL;
+				  /* Should there be a new state set here ? */
+				  return (False);
+			  } else {
+				/* limit not reached try again */
+				++reread_cnt;
+				/*
+				** go back to SOCKET and re do complete html query
+				*/
+				state = SOCKET;
+				close(s);
+				XtFree(html);
+				html = NULL;
+				/*
+				 * FIX ME should probably implement a simple backoff
+				 * delay here, to ensure an already overloaded server
+				 * isn't getting even more overloaded.
+				 */
+				return (False);
+				/*break;*/
+			  }
+		  } else {    /* code not 400 and not 200 */
+		    start = html + num;
+		    if ( (end = strchr ( start, '\r')) == NULL)
+		      end = start + 20;
+		    sprintf ( errmsg, "%s: %d %.*s", 
+			(command!=WORK_NEWS) ? tickGetName(i):"Xquote News",
+			code, end-start, start);
+		    write_status (errmsg, ERR);
+		    reread_cnt = 0;
+		    /* Should there be a new state set here ? */
+	          }
 	  }
 	}
+	    reread_cnt = 0;
         state = PARSE;
-      } else {                  /* MAYBE MORE */
+      } else {  /* status != 0 */                /* MAYBE MORE */
         state = READ;
       }
       return (False);
@@ -432,7 +543,8 @@ int workNet (int command)
           tickSetDetail ( i, findPattern(html) );
         state = TEST;
       } else {
-        aboutText( findNews(html) );
+        if (html)
+          aboutText( findNews(html) );
         state = END;
       }
       XtFree(html);
@@ -443,9 +555,19 @@ int workNet (int command)
 
     case TEST:
       if ( ++i < tickGetNum() ) {
-        state = SOCKET;
+	if (tickGetServer(i) != getCurServer())
+	  state = IDLE;
+	else
+          state = SOCKET;
+	/*
+	 * Get Xquote to update its display whenever it gets a quote,
+	 * not after it got them all.
+	 */
+#if 0
+        detailUpdateMainWindow(DETAIL_UPDATE);
+#endif
       } else {
-        detailUpdateMainWindow ();
+        detailUpdateMainWindow(DETAIL_UPDATE);
         makeTextTape();
         state = UPDATE;
       }
@@ -462,13 +584,14 @@ int workNet (int command)
     default:
       if (!proxy)
         XtFree (serv_machine);
-      sprintf (errmsg, "%s%.*s", version, 
-               80-(int)strlen(version), time_msg);
+
+      strcpy(errmsg, time_msg);
       write_status_line (errmsg);
 
       cylonStart (False);
 
       state = IDLE;
+      i = 0;
       netFinished = True;
       work = (XtWorkProcId) NULL;
       return (True);
@@ -484,24 +607,20 @@ int workNet (int command)
 #undef UPDATE
 #undef END
 
-/* ARGSUSED */
-static void serverNet (Widget w, XtPointer client_data, XtPointer call_data)
-{
-  pServer = (int) client_data;
-}
-
 
 /* ARGSUSED */
 void procNet (Widget w, XtPointer which, XtPointer call_data)
 {
-  static Widget proxy = (Widget) NULL, port;
+  static Widget proxy = (Widget) NULL, port, retries;
   static char *server, *portno;
+  char *s;
 
   if (proxy == (Widget)NULL) {
     proxy = XtNameToWidget (GetTopShell(w),
-                            "NetPane.NetRow.NetProxy.form_1.text_0");
+                            "NetPane.NetRow.NetProxy.form_0.text_0");
     port = XtNameToWidget  (GetTopShell(w),
-                            "NetPane.NetRow.NetProxy.form_1.text_1");
+                            "NetPane.NetRow.NetProxy.form_0.text_1");
+    retries = XtNameToWidget(GetTopShell(w), "NetPane*NetRetries*text_0");
   }
 
   switch ((int) which) {
@@ -522,12 +641,14 @@ void procNet (Widget w, XtPointer which, XtPointer call_data)
 	      } else {
                 setProxy (NULL, "80");
               }
-
 	      /*
-	      ** Current Server
+	      ** RETRIES
 	      */
-              serverCB ((Widget)NULL, (XtPointer)pServer, (XtPointer)NULL);
-
+	      s = XmTextFieldGetString(retries);
+	      if (strlen(s)) {
+		reread_lmt = atoi(s);
+	      }
+	      XtFree(s);
 	    }
             break;
 
@@ -539,25 +660,11 @@ void procNet (Widget w, XtPointer which, XtPointer call_data)
 	      XmTextFieldSetString (proxy, server);
 	    if (portno)
 	      XmTextFieldSetString (port, portno);
-
-            { 
-              Widget menu, cascade;
-              WidgetList buttons;
-              XmString label;
-              int value = getCurServer();
-
-              /* Change menu history */
-              XtVaGetValues (optmenu, XmNsubMenuId, &menu, NULL);
-              XtVaGetValues (menu, XmNchildren, &buttons, NULL);
-              XtVaSetValues (optmenu, XmNmenuHistory, buttons[value], NULL);
-
-              /* Change cascade button label */
-              cascade = XmOptionButtonGadget (optmenu);
-              XtVaGetValues (buttons[value], XmNlabelString, &label, NULL);
-              XtVaSetValues (cascade, XmNlabelString, label, NULL);
-
-              XmUpdateDisplay (cascade);
-            }
+	    s = XtMalloc(10);
+	    sprintf(s, "%d", reread_lmt);
+	    XmTextFieldSetString(retries, s);
+	    XtFree(s);
+	    
             break;
 
     default: break;
@@ -566,8 +673,8 @@ void procNet (Widget w, XtPointer which, XtPointer call_data)
 
 Widget createNetDialog ()
 {
-  char *frames[] =  {"NetSource", "NetProxy"};
-  char *forms[] =   {"form_0", "form_1"};
+  char *frames[] =  {"NetProxy", "NetRetries" };
+  char *forms[] =   {"form_0"};
   char *buttons[] = {"button_0", "button_1"};
 
   Widget dialog, form, pane, row, frame, button, label, text;
@@ -577,7 +684,7 @@ Widget createNetDialog ()
 
   dialog = XtVaCreatePopupShell ("OptionNet", 
                            xmDialogShellWidgetClass,
-                           GetTopShell(Toplevel),
+                           GetTopShell(per->Toplevel),
                            NULL);
 
   pane = XtVaCreateWidget ("NetPane", xmPanedWindowWidgetClass, 
@@ -602,94 +709,78 @@ Widget createNetDialog ()
                           XmNchildType, XmFRAME_TITLE_CHILD,
                           XmNchildVerticalAlignment, XmALIGNMENT_CENTER,
                           NULL);
-     form = XtVaCreateWidget ( forms[num], 
-                            xmFormWidgetClass, frame, 
-                            NULL );
+     form = XtVaCreateWidget (forms[num], xmFormWidgetClass, frame, NULL );
 
      switch (num) {
+     case 0:	/* Proxy */
+     {
+         char *host, port[12];
+	 int portno;
 
-        case 0: {
-		  Widget menu, cascade;
-                  Widget visible = (Widget)NULL;
-		  XmString optlabel;
-		  int servers;
+	 getProxy (&host, &portno);
+	 if (host)
+	   sprintf ( port, "%d", portno );
 
-		  menu = XmCreatePulldownMenu ( form, "NetOptionMenu_pulldown", 
-			     NULL, 0);
-		  optmenu = XmCreateOptionMenu (form, "NetOptionMenu", NULL, 0);
-
-                  /* Add menu items */
-		  for (servers = 0; servers < numServer(); servers++) {
-		    button = XtVaCreateManagedWidget ( getServerTitle(servers),
-		                       xmPushButtonGadgetClass, menu, NULL);
-		    XtAddCallback (button, XmNactivateCallback, serverNet,
-		  	           (XtPointer)servers);
-                    if (servers == getCurServer())
-                      visible = button;
-		  }
-
-                  /* Make server label the one shown */
-		  optlabel = XmStringCreateLocalized (
-				        getServerTitle (getCurServer()));
-		  cascade = XmOptionButtonGadget (optmenu);
-		  XtVaSetValues (cascade, XmNlabelString, optlabel, NULL);
-		  XmStringFree (optlabel);
-
-                  /* Make the active menu item the one shown */
-		  XtVaSetValues (optmenu, 
-		                 XmNsubMenuId, menu,
-                                 XmNmenuHistory, visible, 
-                                 NULL);
-
-		  setServerType (0);
-                  XtManageChild (optmenu);
-		}
-                break;
-
-        case 1: {
-	          char *host, port[12];
-		  int portno;
-
-		  getProxy (&host, &portno);
-		  if (host)
-		    sprintf ( port, "%d", portno );
-
-                  label = XtVaCreateManagedWidget ( "label_0",
+           label = XtVaCreateManagedWidget ( "label_0",
                             xmLabelGadgetClass, form,
                             XmNleftAttachment, XmATTACH_FORM,
                             XmNtopAttachment, XmATTACH_FORM,
                             XmNbottomAttachment, XmATTACH_FORM,
                             NULL );
-                  text = XtVaCreateManagedWidget ( "text_0",
+           text = XtVaCreateManagedWidget ( "text_0",
                             xmTextFieldWidgetClass, form,
                             XmNleftAttachment, XmATTACH_WIDGET,
                             XmNleftWidget, label,
                             XmNtopAttachment, XmATTACH_FORM,
                             XmNbottomAttachment, XmATTACH_FORM,
                             NULL );
-		  if (host)
-		    XmTextFieldSetString (text, host);
+	   if (host)
+	     XmTextFieldSetString (text, host);
 
-                  label = XtVaCreateManagedWidget ( "label_1",
+           label = XtVaCreateManagedWidget ( "label_1",
                             xmLabelGadgetClass, form,
                             XmNleftAttachment, XmATTACH_WIDGET,
                             XmNleftWidget, text,
                             XmNtopAttachment, XmATTACH_FORM,
                             XmNbottomAttachment, XmATTACH_FORM,
                             NULL );
-                  text = XtVaCreateManagedWidget ( "text_1",
+           text = XtVaCreateManagedWidget ( "text_1",
                             xmTextFieldWidgetClass, form,
                             XmNleftAttachment, XmATTACH_WIDGET,
                             XmNleftWidget, label,
                             XmNtopAttachment, XmATTACH_FORM,
                             XmNbottomAttachment, XmATTACH_FORM,
                             NULL );
-		  if (host)
-		    XmTextFieldSetString (text, port);
-		}
-                break;
-
+	   if (host)
+	     XmTextFieldSetString (text, port);
      }
+     break;
+     case 1:	/* Retry count */
+     {
+	char	scount[10];
+	text = XtVaCreateManagedWidget ( "text_0",
+		xmTextFieldWidgetClass, form,
+		XmNrightAttachment, XmATTACH_FORM,
+		XmNtopAttachment, XmATTACH_FORM,
+		XmNbottomAttachment, XmATTACH_FORM,
+		NULL );
+	label = XtVaCreateManagedWidget ( "label_0",
+		xmLabelGadgetClass, form,
+		XmNleftAttachment,	XmATTACH_FORM,
+		XmNtopAttachment,	XmATTACH_FORM,
+		XmNbottomAttachment,	XmATTACH_FORM,
+		XmNrightAttachment,	XmATTACH_WIDGET,
+		XmNrightWidget,		text,
+		XmNrightOffset,		5,
+		XmNalignment,		XmALIGNMENT_BEGINNING,
+		NULL );
+	sprintf(scount, "%d", reread_lmt);
+	XmTextFieldSetString (text, scount);
+     }
+
+	break;
+     }
+
      XtManageChild (form);
 
   }
