@@ -1,28 +1,35 @@
 /*
-  Xquote Copyright (c)1997 Mark Buser,
-  All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-  
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xquote, but
-  modified versions may not be distributed
-  without prior consent of the author.
-  
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 1.5 $ $Date: 1997/10/31 23:09:07 $
-*/
-
+ * Copyright (c) 1997, 2001, Mark Buser.
+ * Copyright (c) 2001, 2003, 2004, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/optupdate.c,v 1.15 2004/05/01 14:09:52 danny Exp $
+ */
 #include <sys/types.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -38,6 +45,10 @@
 #include "xutil.h"
 #include "xquote.h"
 
+#ifndef XQUOTE
+#include "xinvest.h"
+#endif
+
 /* Given we're not updating today, when is the next day and
 ** start time for an update.
 */ 
@@ -60,12 +71,10 @@
         next->tm_min = 0;                              \
         next->tm_sec = 0;
 
-/* Extern */
-extern Widget Toplevel;
 extern XtWorkProcId work;
 extern int netFinished;
 
-/* Globals */
+/* Not sure if this belongs here FIX ME */
 char time_msg[80];
 
 /* Local Globals */
@@ -208,12 +217,12 @@ static void timeUpdate (Widget w, XtPointer client_data, XtPointer call_data)
 void timeMsgUpdate (int command, time_t nextUpdate)
 {
   time_t t = time ((time_t *)NULL);
-  struct tm *now;
+  struct tm *now = localtime(&t);
 
   switch (command) {
     case TRIGGER_START:
          now = localtime (&nextUpdate);
-         strftime (time_msg, XtNumber(time_msg),"  Next Update => %a %X", now);
+         strftime (time_msg, XtNumber(time_msg),"Next update : %x %X", now);
          break;
 
     case TRIGGER_STOP:
@@ -228,7 +237,6 @@ void timeMsgUpdate (int command, time_t nextUpdate)
 /* Schedule next update, considering user selected valid days/times */
 static time_t scheduleUpdate () {
 
-  int dday;
   time_t t, nextUpdate;
   struct tm *curr, *next;
 
@@ -240,7 +248,12 @@ static time_t scheduleUpdate () {
       (days == 0 && curr->tm_wday > 0 && curr->tm_wday < 6)) {  /* M-F */
     /* Today is an update day */
     next = curr;
-    if (curr->tm_hour < start) {
+    if (start == end) {
+      /* Always update */
+      t += (time_t)(minutes * 60);
+      next = localtime(&t);
+      nextUpdate = t;
+    } else if (curr->tm_hour < start) {
       /* Next update is at start time today */
       next->tm_hour = start;
       next->tm_min = 0;
@@ -273,6 +286,16 @@ static time_t scheduleUpdate () {
 }
 
 
+/*
+ * Record the time of last update
+ */
+static time_t	last_update = (time_t) 0;
+void set_current_time()
+{
+	last_update = time((time_t *)0);
+	
+}
+
 /* Start net access, no auto if command is refresh */
 
 /* ARGSUSED */
@@ -282,7 +305,7 @@ void triggerUpdate (int command, XtIntervalId z)
 
   /* Busy, reschedule in 1 minute */
   if (command != TRIGGER_STOP && netFinished == False) {
-    timer = XtAppAddTimeOut (XtWidgetToApplicationContext(Toplevel),
+    timer = XtAppAddTimeOut (XtWidgetToApplicationContext(per->Toplevel),
                  1L*60L*1000L, (XtTimerCallbackProc) triggerUpdate, 
 		 (XtPointer) command);
     return;
@@ -309,11 +332,11 @@ void triggerUpdate (int command, XtIntervalId z)
         if (timer && timer != z)    /* should be no outstanding timers */
           XtRemoveTimeOut (timer);
         timer = (XtIntervalId) NULL;
-        work = XtAppAddWorkProc (XtWidgetToApplicationContext(Toplevel), 
+        work = XtAppAddWorkProc (XtWidgetToApplicationContext(per->Toplevel), 
                     (XtWorkProc)workNet, (XtPointer)WORK_NORMAL);
 
         if (running) {
-          timer = XtAppAddTimeOut (XtWidgetToApplicationContext(Toplevel),
+          timer = XtAppAddTimeOut (XtWidgetToApplicationContext(per->Toplevel),
                     (nextUpdate-t)*1000L, (XtTimerCallbackProc)triggerUpdate, 
 		    (XtPointer)TRIGGER_START);
           timeMsgUpdate (command, nextUpdate);
@@ -325,19 +348,18 @@ void triggerUpdate (int command, XtIntervalId z)
       if (tickGetNum() == 0) 
         write_status ("There are no ticker symbols to fetch.", ERR);
       else
-        work = XtAppAddWorkProc (XtWidgetToApplicationContext(Toplevel), 
+        work = XtAppAddWorkProc (XtWidgetToApplicationContext(per->Toplevel), 
                               (XtWorkProc)workNet, (XtPointer)WORK_NORMAL);
       break;
 
     case TRIGGER_NEWS:
-      work = XtAppAddWorkProc (XtWidgetToApplicationContext(Toplevel), 
+      work = XtAppAddWorkProc (XtWidgetToApplicationContext(per->Toplevel), 
                                (XtWorkProc)workNet, (XtPointer)WORK_NEWS);
       break;
 
     default: break;
   }
-  sprintf (errmsg, "%s%.*s", version, 
-           80-(int)strlen(version), time_msg);
+  strcpy(errmsg, time_msg);
   write_status_line (errmsg);
 }
 
@@ -395,8 +417,7 @@ static void procUpdate (Widget w, XtPointer which, XtPointer call_data)
                 timeMsgUpdate (TRIGGER_STOP, nextUpdate);
               }
             }
-            sprintf (errmsg, "%s%.*s", version, 
-                     80-(int)strlen(version), time_msg);
+	    sprintf(errmsg, "");
             write_status_line (errmsg);
 
 	  }
@@ -458,7 +479,7 @@ Widget createUpdateDialog ()
 
   dialog = XtVaCreatePopupShell ("OptionUpdate", 
                            xmDialogShellWidgetClass,
-                           GetTopShell(Toplevel),
+                           GetTopShell(per->Toplevel),
                            NULL );
 
   pane = XtVaCreateWidget ("UpdatePane", xmPanedWindowWidgetClass, dialog,
