@@ -1,3 +1,5 @@
+"Tape.foreground:			white",
+"*XmDrawingArea.foreground:		white",
 "*background:				#6e91aa",
 "*highlightColor:			#cc686f",
 "*XmDialogShell*background:		#8b979f",
@@ -9,3 +11,4 @@
 "*XmDialogShell*XmList.background:	#e1ebe1",
 "*XmDialogShell*XmText.background:	#e1ebe1",
 "*XmDialogShell*XmTextField.background:	#e1ebe1",
+"*Container.gridType:			grid_cell_shadow",
